import Combine
import ComposableArchitecture
import ElixxirDAppsSDK
import ErrorFeature
import LandingFeature
import SessionFeature
import SwiftUI

@main
struct App: SwiftUI.App {
  var body: some Scene {
    WindowGroup {
      AppView(store: Store(
        initialState: AppState(),
        reducer: appReducer,
        environment: .live()
      ))
    }
  }
}

extension AppEnvironment {
  static func live() -> AppEnvironment {
    let cMixSubject = CurrentValueSubject<CMix?, Never>(nil)
    let mainScheduler = DispatchQueue.main.eraseToAnyScheduler()
    let bgScheduler = DispatchQueue(
      label: "xx.network.dApps.ExampleApp.bg",
      qos: .background
    ).eraseToAnyScheduler()

    return AppEnvironment(
      makeId: UUID.init,
      hasCMix: { cMixSubject.map { $0 != nil }.eraseToAnyPublisher() },
      mainScheduler: mainScheduler,
      landing: LandingEnvironment(
        cMixManager: .live(
          passwordStorage: .keychain
        ),
        setCMix: { cMixSubject.value = $0 },
        bgScheduler: bgScheduler,
        mainScheduler: mainScheduler,
        error: ErrorEnvironment()
      ),
      session: SessionEnvironment(
        getCMix: { cMixSubject.value },
        bgScheduler: bgScheduler,
        mainScheduler: mainScheduler,
        error: ErrorEnvironment()
      )
    )
  }
}
